package lrs.visitor;

import lrs.*;
import fp.*;
import java.util.*;


/**
 * Determines if a given Comparable object is contained in the list.
 * The list does not need to contain Comparables bacause 
 * comp.compareTo(listElement) is used to equality testing.
 * Exhibits "short-circuit" behavior in that if
 * a matching element is found in the list 
 * the processing of the list is 
 * immediately terminated and true is returned.
 * If the list is empty, false is returned.
 * The original list is unchanged.
 */
public class IsInLRS_SOL implements IAlgo {
  
  public static final IsInLRS_SOL Singleton = new IsInLRS_SOL();
  
  private IsInLRS_SOL() {}
  
  public Object emptyCase(LRStruct host, Object comp) {
    return false;
  }
  
  public Object nonEmptyCase(LRStruct host, final Object comp) {
    return host.execute(OrMapLRS.Singleton, new ILambda() {
      public Object apply(Object...xs) {
        return 0==((Comparable)comp).compareTo(xs[0]);
      }
    });
  }
}