package lrs.visitor;
import lrs.*;
import fp.*;

/**
 * Maps an ILambda across every element of the list.
 * Stops processing the list if the lambda returns false.
 * The list is affected only by way of side-effects from the 
 * lambda.  Thus the mapping may or may not cause mutation in the
 * list.  The lambda is given the whole list at each node, 
 * not just the data.
 * empty case: map(empty, f, b) = empty;
 * non-empty case: map({x1, x2, x3 ...}, f, b) = {f(x1, b), f(x2,b), f(x3,b), ...};
 */
public class MapLRS implements IAlgo {
  ILambda f;
  
  public MapLRS(ILambda f) {
    this.f = f;
  }
  
  public Object emptyCase(LRStruct host, Object inp) {
    return host;
  }
  public Object nonEmptyCase(LRStruct host, Object inp) {
    if((Boolean)f.apply(host, inp)) {
      host.getRest().execute(this, inp);
    }
    return host;
  }

}
  
